// Forlix Piratewars
// http://forlix.org/, df@forlix.org
//
// Copyright (c) 2008-2013 Dominik Friedrichs

#include <sourcemod>
#include <sdktools>
#include <cstrike>

#pragma semicolon 1

#define PLUGIN_VERSION            "1.42"
#define PLUGIN_VERSION_CVAR       "forlix_piratewars_version"

#define PLUGIN_COMMAND_DEADTIME   0.2

#define HIDEHUD_WEAPONSELECTION   (1<<0)

#define TRANSLATION_FILE          "forlix_piratewars.phrases.txt"

#define MAX_HANDLERS              32

#define TRIGGER_ENTREF            0
#define GAMEUI_ENTREF             1
#define HANDLER_TYPE              2
#define HANDLER_TEAM              3
#define CLIENT_INDEX              4
#define WEAPON_ENTREF             5

#define TYPE_CANNON               (1<<0)
#define TYPE_BOAT                 (1<<1)

#define TEAM_T                    (1<<0)
#define TEAM_CT                   (1<<1)

// SourceMod defines for these are wrong
#define FL_ONTRAIN                (1<<4)
#define FL_ATCONTROLS             (1<<7)

public Plugin:myinfo =
{
  name = "Forlix Piratewars",
  author = "Forlix (Dominik Friedrichs)",
  description = "Bugfixes and enhancements for mg_piratewars",
  version = PLUGIN_VERSION,
  url = "http://forlix.org/"
};

new p_intrigger_ref[MAXPLAYERS+1] = {INVALID_ENT_REFERENCE, ...};
new p_hints[MAXPLAYERS+1];

new sunkenflags = 0;
new Float:sinktime = -1.0;

new gameui_handler[MAX_HANDLERS][6];
// gameui_handler structure:
// [handler_idx][TRIGGER_ENTREF]  trigger entity reference
// [handler_idx][GAMEUI_ENTREF]   game_ui entity reference
// [handler_idx][HANDLER_TYPE]    handler type flag (TYPE_CANNON or TYPE_BOAT)
// [handler_idx][HANDLER_TEAM]    team side flag (TEAM_T or TEAM_CT)
// [handler_idx][CLIENT_INDEX]    client index on, if any
// [handler_idx][WEAPON_ENTREF]   client weapon entity reference buffer


public OnPluginStart()
{
  RegConsoleCmd("use", Command_Use);

  HookEvent("player_spawn", Event_PlayerSpawn);
  HookEvent("round_start", Event_RoundStart);

  new Handle:version_cvar = CreateConVar(PLUGIN_VERSION_CVAR,
  PLUGIN_VERSION,
  "Forlix Piratewars plugin version",
  FCVAR_PLUGIN|FCVAR_SPONLY|FCVAR_NOTIFY|FCVAR_PRINTABLEONLY);

  SetConVarString(version_cvar, PLUGIN_VERSION, false, false);

  return;
}


public OnMapStart()
{
  decl String:currentmap[32];
  GetCurrentMap(currentmap, sizeof(currentmap));

  if(StrEqual(currentmap, "mg_piratewars")
  || StrEqual(currentmap, "mg_piratewars_2010"))
    LoadTranslations(TRANSLATION_FILE);
  else
    UnloadWithMessage("Forlix Piratewars unloads (Map not supported)");

  return;
}


public OnClientConnected(client)
{
  p_intrigger_ref[client] = INVALID_ENT_REFERENCE;
  p_hints[client] = 0;

  return;
}


public OnGameFrame()
{
  static p_lastbtnstate[MAXPLAYERS+1];

  for(new client = 1; client <= MaxClients; client++)
  if(IsClientInGame(client)
  && IsPlayerAlive(client))
  {
    new btnstate = GetClientButtons(client);
    new btnchange = btnstate^p_lastbtnstate[client];

    p_lastbtnstate[client] = btnstate;

    if((btnchange & IN_USE)
    && (btnstate & IN_USE))
      Player_Use(client);
  }

  return;
}


public Action:Command_Use(client, args)
{
  if(IsClientInGame(client)
  && GetEntProp(client, Prop_Send, "m_iHideHUD") & HIDEHUD_WEAPONSELECTION)
  // client has no weapon selection, so block the use command
    return(Plugin_Handled);

  return(Plugin_Continue);
}


public Event_PlayerSpawn(Handle:event,
                         const String:Event_type[],
                         bool:dontBroadcast)
{
  new client = GetClientOfUserId(GetEventInt(event, "userid"));

  p_intrigger_ref[client] = INVALID_ENT_REFERENCE;
  return;
}


public Event_RoundStart(Handle:event,
                        const String:Event_type[],
                        bool:dontBroadcast)
{
  ReHookEntities();
  sunkenflags = 0;
  sinktime = -1.0;

  for(new i = 1; i <= 4; i++)
  // make some modifications to the 4 rescue boats
  {
    decl String:dingy_tn_orig[16];
    decl String:dingy_tn[16];
    decl String:dingy_ui_tn[16];

    decl String:thruster_forward_tn[32];
    decl String:thruster_reverse_tn[32];

    decl String:buf[64];

    // format targetnames of relevant entities
    Format(dingy_tn_orig, sizeof(dingy_tn_orig), "dingy%u", i);
    Format(dingy_tn, sizeof(dingy_tn), "_dingy%u", i);
    Format(dingy_ui_tn, sizeof(dingy_ui_tn), "dingy%u_ui", i);

    Format(thruster_forward_tn, sizeof(thruster_forward_tn), "thruster%u_forward", i);
    Format(thruster_reverse_tn, sizeof(thruster_reverse_tn), "thruster%u_reverse", i);

    new dingy = FindEntityByTargetname(-1, dingy_tn_orig, "func_physbox");
    new dingy_ui = FindEntityByTargetname(-1, dingy_ui_tn, "game_ui");
    new fwd_thruster = FindEntityByTargetname(-1, thruster_forward_tn, "phys_thruster");

    if(dingy <= 0
    || dingy_ui <= 0
    || fwd_thruster <= 0)
    {
      LogError("Entity not found (DINGY %i <%s>, DINGY_UI %i <%s>, FWD_THRUSTER %i <%s>)",
               dingy, dingy_tn_orig, dingy_ui, dingy_ui_tn, fwd_thruster, thruster_forward_tn);
      continue;
    }

    // set spawnflags to make boat indestructible
    DispatchKeyValue(dingy, "spawnflags", "33792");

    // change targetname of the physbox to prevent enablemotion when touching the trigger
    // instead, add output to the game_ui entity to enablemotion on first playeron
    DispatchKeyValue(dingy, "targetname", dingy_tn);

    // only needs to fire once (enablemotion when first player starts using)
    Format(buf, sizeof(buf), "PlayerOn %s,EnableMotion,,0,1", dingy_tn);
    SetVariantString(buf);
    AcceptEntityInput(dingy_ui, "AddOutput");

    // add reverse-driving capability
    // get position and angles of forward_thruster
    new Float:thruster_origin[3];
    new Float:thruster_angles[3];

    GetEntPropVector(fwd_thruster, Prop_Data, "m_vecAbsOrigin", thruster_origin);
    GetEntPropVector(fwd_thruster, Prop_Data, "m_angAbsRotation", thruster_angles);

    // reverse angle
    if((thruster_angles[1] -= 180) < 0)
      thruster_angles[1] += 360;

    Format(buf, sizeof(buf),
    "%.f %.f %.f",
    thruster_angles[0],
    thruster_angles[1],
    thruster_angles[2]);

    // spawn reverse thruster at the same origin but at reversed angle
    new rev_thruster = CreateEntityByName("phys_thruster", -1);

    DispatchKeyValue(rev_thruster, "spawnflags", "14");
    DispatchKeyValue(rev_thruster, "targetname", thruster_reverse_tn);

    DispatchKeyValue(rev_thruster, "force", "40000");
    DispatchKeyValue(rev_thruster, "forcetime", "0");
    DispatchKeyValue(rev_thruster, "angles", buf);

    DispatchKeyValue(rev_thruster, "attach1", dingy_tn);
    SetEntPropEnt(rev_thruster, Prop_Data, "m_attachedObject", dingy);

    TeleportEntity(rev_thruster, thruster_origin, NULL_VECTOR, NULL_VECTOR);
    DispatchSpawn(rev_thruster);

    // add game_ui outputs to control the newly added thruster
    Format(buf, sizeof(buf), "PressedBack thruster%u_reverse,Activate,,0,-1", i);
    SetVariantString(buf);
    AcceptEntityInput(dingy_ui, "AddOutput");

    Format(buf, sizeof(buf), "UnpressedBack thruster%u_reverse,Deactivate,,0,-1", i);
    SetVariantString(buf);
    AcceptEntityInput(dingy_ui, "AddOutput");

    // add outputs so thrusters always turn off when player stops driving the boat
    // prevents boat from driving on and on without a driver
    Format(buf, sizeof(buf), "PlayerOff thruster%u_forward,Deactivate,,0,-1", i);
    SetVariantString(buf);
    AcceptEntityInput(dingy_ui, "AddOutput");

    Format(buf, sizeof(buf), "PlayerOff thruster%u_left,Deactivate,,0,-1", i);
    SetVariantString(buf);
    AcceptEntityInput(dingy_ui, "AddOutput");

    Format(buf, sizeof(buf), "PlayerOff thruster%u_right,Deactivate,,0,-1", i);
    SetVariantString(buf);
    AcceptEntityInput(dingy_ui, "AddOutput");

    Format(buf, sizeof(buf), "PlayerOff thruster%u_reverse,Deactivate,,0,-1", i);
    SetVariantString(buf);
    AcceptEntityInput(dingy_ui, "AddOutput");
  }

  // first call to InitGameUIHandler with reset = true to delete all previous handlers

  // why find the triggers by origin? because mapper did not add targetnames
  // why are most of them trigger_push instead of trigger_multiple? i have no idea...

  // each gameui station consists of a trigger brush in which the player
  // can press use to activate the gameui, and the gameui entity itself

  decl String:currentmap[32];
  GetCurrentMap(currentmap, sizeof(currentmap));

  if(StrEqual(currentmap, "mg_piratewars"))
  {
    new index = -1;
    // fixes the ships not breaking upon being hit after the CS:S OB update

    while((index = FindEntityByTargetname(index, "pw_break_t", "func_breakable")) > 0)
      DispatchKeyValue(index, "spawnflags", "512");

    while((index = FindEntityByTargetname(index, "pw_break_ct", "func_breakable")) > 0)
      DispatchKeyValue(index, "spawnflags", "512");

    // T side gameui handlers
    InitGameUIHandler(Float:{1674.15, 1180.77, -20.0}, "trigger_push", "pw_cannon101_ui", TYPE_CANNON, TEAM_T, true);
    InitGameUIHandler(Float:{1424.63, 1180.6, -20.0}, "trigger_push", "pw_cannon2_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{370.5, 1179.0, -177.0}, "trigger_push", "pw_cannon01_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{93.5, 1179.0, -177.0}, "trigger_push", "pw_cannon3_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{-319.72, 1226.69, -169.0}, "trigger_push", "pw_cannon100_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{-675.59, 1348.22, -177.0}, "trigger_push", "pw_cannon4_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{131.5, 1181.0, -395.0}, "trigger_push", "pw_cannon5_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{438.5, 1183.0, -395.0}, "trigger_push", "pw_cannon6_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{757.5, 1183.0, -387.0}, "trigger_push", "pw_cannon7_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{1072.5, 1181.0, -387.0}, "trigger_push", "pw_cannon8_ui", TYPE_CANNON, TEAM_T);

    InitGameUIHandler(Float:{389.0, 1956.5, -437.0}, "trigger_multiple", "dingy1_ui", TYPE_BOAT, TEAM_T);
    InitGameUIHandler(Float:{195.0, 1958.5, -437.0}, "trigger_multiple", "dingy3_ui", TYPE_BOAT, TEAM_T);

    // CT side gameui handlers
    InitGameUIHandler(Float:{1745.35, -1060.45, -20.0}, "trigger_push", "pw_cannon102_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{1473.41, -1057.24, -20.0}, "trigger_push", "pw_cannon12_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{421.5, -1054.01, -177.0}, "trigger_push", "pw_cannon11_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{144.5, -1054.01, -177.0}, "trigger_push", "pw_cannon13_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{-266.72, -1101.94, -169.0}, "trigger_push", "pw_cannon104_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{-624.59, -1223.23, -177.0}, "trigger_push", "pw_cannon14_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{182.5, -1056.01, -395.0}, "trigger_push", "pw_cannon15_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{489.5, -1058.01, -395.0}, "trigger_push", "pw_cannon16_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{808.5, -1058.01, -387.0}, "trigger_push", "pw_cannon17_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{1123.5, -1056.01, -387.0}, "trigger_push", "pw_cannon18_ui", TYPE_CANNON, TEAM_CT);

    InitGameUIHandler(Float:{447.0, -1836.5, -437.0}, "trigger_multiple", "dingy2_ui", TYPE_BOAT, TEAM_CT);
    InitGameUIHandler(Float:{239.0, -1834.5, -437.0}, "trigger_multiple", "dingy4_ui", TYPE_BOAT, TEAM_CT);
  }
  else
  if(StrEqual(currentmap, "mg_piratewars_2010"))
  {
    // T side gameui handlers
    InitGameUIHandler(Float:{1377.06, 1097.77, 427.0}, "trigger_push", "pw_cannon101_ui", TYPE_CANNON, TEAM_T, true);
    InitGameUIHandler(Float:{1127.54, 1097.6, 427.0}, "trigger_push", "pw_cannon2_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{73.41, 1096.0, 270.0}, "trigger_push", "pw_cannon01_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{-203.59, 1096.0, 270.0}, "trigger_push", "pw_cannon3_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{-616.81, 1143.69, 278.0}, "trigger_push", "pw_cannon100_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{-972.68, 1265.22, 270.0}, "trigger_push", "pw_cannon4_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{-165.59, 1098.0, 52.0}, "trigger_push", "pw_cannon5_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{141.41, 1100.0, 52.0}, "trigger_push", "pw_cannon6_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{460.41, 1100.0, 60.0}, "trigger_push", "pw_cannon7_ui", TYPE_CANNON, TEAM_T);
    InitGameUIHandler(Float:{775.41, 1098.0, 60.0}, "trigger_push", "pw_cannon8_ui", TYPE_CANNON, TEAM_T);

    InitGameUIHandler(Float:{91.91, 1873.5, 10.0}, "trigger_multiple", "dingy1_ui", TYPE_BOAT, TEAM_T);
    InitGameUIHandler(Float:{-102.09, 1875.5, 10.0}, "trigger_multiple", "dingy3_ui", TYPE_BOAT, TEAM_T);

    // CT side gameui handlers
    InitGameUIHandler(Float:{1448.26, -1143.45, 427.0}, "trigger_push", "pw_cannon102_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{1176.32, -1140.24, 427.0}, "trigger_push", "pw_cannon12_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{124.41, -1137.01, 270.0}, "trigger_push", "pw_cannon11_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{-152.59, -1137.01, 270.0}, "trigger_push", "pw_cannon13_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{-563.81, -1184.94, 278.0}, "trigger_push", "pw_cannon104_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{-921.68, -1306.23, 270.0}, "trigger_push", "pw_cannon14_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{-114.59, -1139.01, 52.0}, "trigger_push", "pw_cannon15_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{192.41, -1141.01, 52.0}, "trigger_push", "pw_cannon16_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{511.41, -1141.01, 60.0}, "trigger_push", "pw_cannon17_ui", TYPE_CANNON, TEAM_CT);
    InitGameUIHandler(Float:{826.41, -1139.01, 60.0}, "trigger_push", "pw_cannon18_ui", TYPE_CANNON, TEAM_CT);

    InitGameUIHandler(Float:{149.91, -1919.5, 10.0}, "trigger_multiple", "dingy2_ui", TYPE_BOAT, TEAM_CT);
    InitGameUIHandler(Float:{-58.09, -1917.5, 10.0}, "trigger_multiple", "dingy4_ui", TYPE_BOAT, TEAM_CT);
  }

  return;
}


InitGameUIHandler(const Float:trig_origin[3],
                  const String:trig_class[],
                  const String:gameui_name[],
                  gameui_type,
                  team_side,
                  bool:reset=false)
{
  static handleinc = 0;

  if(reset)
  {
    // reset storage index to zero
    handleinc = 0;

    for(new i = 0; i < MAX_HANDLERS; i++)
    // clear all gameui handlers
      gameui_handler[i][TRIGGER_ENTREF] = INVALID_ENT_REFERENCE;
  }

  if(handleinc >= MAX_HANDLERS)
  // handler list overflowed
    return;

  new trig_idx = FindEntityByOrigin(trig_origin, trig_class);
  new gameui_idx = FindEntityByTargetname(-1, gameui_name, "game_ui");

  if(trig_idx <= 0
  || gameui_idx <= 0)
  {
    LogError("Entity not found (TRIG %i {%.1f, %.1f, %.1f}, GAMEUI %i <%s>)",
             trig_idx, trig_origin[0], trig_origin[1], trig_origin[2], gameui_idx, gameui_name);
    return;
  }

  // clear the targetname to make gameui unreachable for other entities
  DispatchKeyValue(gameui_idx, "targetname", "");

  // prevent gameui from disconnecting the player on use and jump
  // and dont let it freeze the player or hide his weapon as that
  // code is buggy and we rather handle this stuff ourselves
  DispatchKeyValue(gameui_idx, "spawnflags", "0");

  gameui_handler[handleinc][TRIGGER_ENTREF] = EntIndexToEntRef(trig_idx);
  gameui_handler[handleinc][GAMEUI_ENTREF] = EntIndexToEntRef(gameui_idx);

  gameui_handler[handleinc][HANDLER_TYPE] = gameui_type;
  gameui_handler[handleinc][HANDLER_TEAM] = team_side;

  gameui_handler[handleinc][CLIENT_INDEX] = 0;
  gameui_handler[handleinc][WEAPON_ENTREF] = INVALID_ENT_REFERENCE;

  handleinc++;
  return;
}


public Event_StartTouch(const String:output[],
                        caller,
                        activator,
                        Float:delay)
{
  new caller_ref = EntIndexToEntRef(caller);

  if(activator <= 0
  || activator > MaxClients
  || !IsClientInGame(activator)
  || p_intrigger_ref[activator] != INVALID_ENT_REFERENCE)
    return;

  new h = FindGameUIHandlerOfTrigger(caller_ref);

  if(h < 0)
  // gameui handler not found
    return;

  // store the trigger the client started touching
  p_intrigger_ref[activator] = caller_ref;

  new gameuitype = gameui_handler[h][HANDLER_TYPE];

  if(p_hints[activator] & gameuitype)
  // player has already seen the hint for this gameuitype, skip
    return;

  switch(gameuitype)
  {
    case TYPE_CANNON:
      PrintHintText(activator, "%t", "HINT_CANNON");

    case TYPE_BOAT:
      PrintHintText(activator, "%t", "HINT_BOAT");
  }

  p_hints[activator] |= gameuitype;
  return;
}


public Event_EndTouch(const String:output[],
                      caller,
                      activator,
                      Float:delay)
{
  new caller_ref = EntIndexToEntRef(caller);

  if(activator <= 0
  || activator > MaxClients
  || p_intrigger_ref[activator] == INVALID_ENT_REFERENCE
  || caller_ref != p_intrigger_ref[activator])
  // allow clients who are not ingame in case
  // someone disconnects within a trigger
    return;

  new h = FindGameUIHandlerOfTrigger(caller_ref);

  if(h < 0)
  // gameui handler not found
    return;

  // clear stored trigger index
  // and try to deactivate the gameui
  p_intrigger_ref[activator] = INVALID_ENT_REFERENCE;
  DeactivateGameUI(h, activator);

  return;
}


public Event_ShipSink(const String:output[],
                      caller,
                      activator,
                      Float:delay)
{
  decl String:targetname[16];
  targetname[0] = '\0';

  GetEntPropString(caller, Prop_Data, "m_iName", targetname, sizeof(targetname));
  new sinkingteam = 0;

  if(StrEqual(targetname, "pw_sink1"))
    sinkingteam = TEAM_T;
  else
  if(StrEqual(targetname, "pw_sink2"))
    sinkingteam = TEAM_CT;
  else
    return;

  if(sinktime < 0)
    sinktime = GetTickedTime();

  // add sinking teams flag to sunkenflags
  sunkenflags |= sinkingteam;

  for(new i = 0; i < MAX_HANDLERS; i++)
  if(gameui_handler[i][TRIGGER_ENTREF] != INVALID_ENT_REFERENCE
  && gameui_handler[i][HANDLER_TYPE] == TYPE_CANNON
  && gameui_handler[i][HANDLER_TEAM] == sinkingteam)
  // deactivate all cannon users on sinking ship
    DeactivateGameUI(i, gameui_handler[i][CLIENT_INDEX]);

  return;
}


Player_Use(client)
{
  // ignore presses within defined deadtime
  static Float:time_nc[MAXPLAYERS+1];
  new Float:time_c = GetTickedTime();

  if(time_c < time_nc[client])
    return;

  time_nc[client] = time_c + PLUGIN_COMMAND_DEADTIME;

  if(p_intrigger_ref[client] == INVALID_ENT_REFERENCE)
  // player is not in a trigger
    return;

  // find the gameui handler
  new h = FindGameUIHandlerOfTrigger(p_intrigger_ref[client]);

  if(h < 0)
  // gameui handler not found
    return;

  // try to toggle the gameui
  ActivateGameUI(h, client)
  || DeactivateGameUI(h, client);

  return;
}


bool:ActivateGameUI(handler_idx,
                    client)
{
  if(gameui_handler[handler_idx][CLIENT_INDEX] > 0
  || (gameui_handler[handler_idx][HANDLER_TYPE] == TYPE_CANNON
  &&  gameui_handler[handler_idx][HANDLER_TEAM] & sunkenflags))
  // gameui is already activated
  // or its a cannon and the teams ship has sunk
    return(false);

  AcceptEntityInput(gameui_handler[handler_idx][GAMEUI_ENTREF],
                    "Activate",
                    client,
                    gameui_handler[handler_idx][GAMEUI_ENTREF]);

  gameui_handler[handler_idx][CLIENT_INDEX] = client;

  // freeze player and hide weapon selection
  SetEntPropFlags(client, Prop_Send, "m_fFlags", FL_ONTRAIN|FL_ATCONTROLS);
  SetEntPropFlags(client, Prop_Send, "m_iHideHUD", HIDEHUD_WEAPONSELECTION);

  // store off players current weapon and then take it away
  new activeweapon = GetEntPropEnt(client, Prop_Send, "m_hActiveWeapon");

  if(activeweapon > 0
  && IsValidEdict(activeweapon))
  {
    RemovePlayerItem(client, activeweapon);
    gameui_handler[handler_idx][WEAPON_ENTREF] = EntIndexToEntRef(activeweapon);
  }
  else
    gameui_handler[handler_idx][WEAPON_ENTREF] = INVALID_ENT_REFERENCE;

  return(true);
}


bool:DeactivateGameUI(handler_idx,
                      client)
{
  if(gameui_handler[handler_idx][CLIENT_INDEX] <= 0
  || gameui_handler[handler_idx][CLIENT_INDEX] != client)
  // gameui is not activated or the
  // given client is not the user of it
    return(false);

  AcceptEntityInput(gameui_handler[handler_idx][GAMEUI_ENTREF],
                    "Deactivate",
                    client,
                    gameui_handler[handler_idx][GAMEUI_ENTREF]);

  gameui_handler[handler_idx][CLIENT_INDEX] = 0;

  if(IsClientInGame(client))
  // unfreeze player and unhide weapon selection
  {
    ClrEntPropFlags(client, Prop_Send, "m_fFlags", FL_ONTRAIN|FL_ATCONTROLS);
    ClrEntPropFlags(client, Prop_Send, "m_iHideHUD", HIDEHUD_WEAPONSELECTION);
  }

  new storedweapon = gameui_handler[handler_idx][WEAPON_ENTREF];

  if(storedweapon != INVALID_ENT_REFERENCE
  && IsValidEdict(storedweapon))
  // we have a valid weapon stored for this player
  {
    if(IsClientInGame(client)
    && IsPlayerAlive(client))
    // give the weapon back
      EquipPlayerWeapon(client, storedweapon);
    else
    // player left or died, delete the weapon
      RemoveEdict(storedweapon);
  }

  gameui_handler[handler_idx][WEAPON_ENTREF] = INVALID_ENT_REFERENCE;

  return(true);
}


SetEntPropFlags(entity,
                PropType:type,
                const String:prop[],
                flags)
{
  SetEntProp(entity, type, prop,
             GetEntProp(entity, type, prop)|flags);
  return;
}


ClrEntPropFlags(entity,
                PropType:type,
                const String:prop[],
                flags)
{
  SetEntProp(entity, type, prop,
             GetEntProp(entity, type, prop)&(~flags));
  return;
}


FindGameUIHandlerOfTrigger(trigger_ref)
{
  for(new i = 0;
      i < MAX_HANDLERS && gameui_handler[i][TRIGGER_ENTREF] != INVALID_ENT_REFERENCE;
      i++)
  if(gameui_handler[i][TRIGGER_ENTREF] == trigger_ref)
    return(i);

  return(-1);
}


FindEntityByTargetname(startindex,
                       const String:targetname[],
                       const String:classname[])
{
  decl String:namebuf[32];
  new index = startindex;
  namebuf[0] = '\0';

  while(strcmp(namebuf, targetname) != 0
    && (index = FindEntityByClassname(index, classname)) != -1)
    GetEntPropString(index, Prop_Data, "m_iName", namebuf, sizeof(namebuf));

  return(index);
}


FindEntityByOrigin(const Float:reforigin[3],
                   const String:classname[])
{
  new Float:originbuf[3];
  new index = -1;

  decl reforigin_rounded[3];
  reforigin_rounded[0] = RoundToNearest(reforigin[0]);
  reforigin_rounded[1] = RoundToNearest(reforigin[1]);
  reforigin_rounded[2] = RoundToNearest(reforigin[2]);

  while( !(RoundToNearest(originbuf[0]) == reforigin_rounded[0]
        && RoundToNearest(originbuf[1]) == reforigin_rounded[1]
        && RoundToNearest(originbuf[2]) == reforigin_rounded[2])
    && (index = FindEntityByClassname(index, classname)) != -1)
    GetEntPropVector(index, Prop_Data, "m_vecAbsOrigin", originbuf);

  return(index);
}


ReHookEntities()
{
  UnhookEntityOutput("trigger_multiple", "OnStartTouch", Event_StartTouch);
  UnhookEntityOutput("trigger_multiple", "OnEndTouch", Event_EndTouch);
  UnhookEntityOutput("trigger_push", "OnStartTouch", Event_StartTouch);
  UnhookEntityOutput("trigger_push", "OnEndTouch", Event_EndTouch);
  UnhookEntityOutput("func_door", "OnOpen", Event_ShipSink);

  HookEntityOutput("trigger_multiple", "OnStartTouch", Event_StartTouch);
  HookEntityOutput("trigger_multiple", "OnEndTouch", Event_EndTouch);
  HookEntityOutput("trigger_push", "OnStartTouch", Event_StartTouch);
  HookEntityOutput("trigger_push", "OnEndTouch", Event_EndTouch);
  HookEntityOutput("func_door", "OnOpen", Event_ShipSink);

  return;
}


UnloadWithMessage(const String:message[])
{
  PrintToServer("%s", message);
  ServerCommand("sm plugins unload forlix_piratewars");

  return;
}
